from __future__ import annotations

__version__ = "0.12.11"


def qualified_version() -> str:
    """Get the qualified version of this module."""

    return f"Meilisearch DocsScraper (v{__version__})"
